package com.logicbig.example;

import javax.persistence.*;
import java.util.HashMap;
import java.util.Map;

public class ExampleMain {

    public static void main(String[] args) {
        EntityManagerFactory emf = Persistence.createEntityManagerFactory("example-unit");
        try {
            persistEntity(emf);
            findItemQty(emf);
            findItemQtyEntry(emf);
            findCustomerByItemName(emf);
        } finally {
            emf.close();
        }
    }

    private static void persistEntity(EntityManagerFactory emf) {
        System.out.println("-- Persisting entities --");
        EntityManager em = emf.createEntityManager();

        Customer c1 = new Customer();
        c1.setName("Lindsey Craft");
        Map<String, Integer> itemPurchased = new HashMap<>();
        itemPurchased.put("XYZ Blender", 1);
        itemPurchased.put("ZZZ Beer Glass", 4);
        c1.setItemQtyMap(itemPurchased);
        System.out.println(c1);

        Customer c2 = new Customer();
        c2.setName("Morgan Philips");
        Map<String, Integer> orderMap2 = new HashMap<>();
        orderMap2.put("AA Glass Cleaner",3);
        c2.setItemQtyMap(orderMap2);
        System.out.println(c2);

        em.getTransaction().begin();
        em.persist(c1);
        em.persist(c2);
        em.getTransaction().commit();
        em.close();
    }

    private static void findItemQty(EntityManagerFactory emf) {
        System.out.println("-- Loading items and keys --");
        EntityManager em = emf.createEntityManager();
        TypedQuery<Tuple> query =
                em.createQuery("SELECT c.name, KEY(map), VALUE(map) "
                        + "FROM Customer c JOIN c.itemQtyMap map", Tuple.class);
        query.getResultList().forEach(
                t -> System.out.printf("Customer: %s, Item: %s, Qty: %s%n", t.get(0), t.get(1), t.get(2)));
    }

    private static void findItemQtyEntry(EntityManagerFactory emf) {
        System.out.println("-- Loading item/keys entries  --");
        EntityManager em = emf.createEntityManager();
        TypedQuery<Map.Entry> query =
                em.createQuery("SELECT ENTRY(c.itemQtyMap) FROM Customer c", Map.Entry.class);
        query.getResultList().forEach(entry -> System.out.printf("Item: %s, Qty: %s%n",
                entry.getKey(), entry.getValue()));
    }

    private static void findCustomerByItemName(EntityManagerFactory emf) {
        System.out.println("-- Loading  customers where item name like %Beer% --");
        EntityManager em = emf.createEntityManager();
        TypedQuery<Customer> query =
                em.createQuery("SELECT c FROM Customer c"
                        + " WHERE KEY(c.itemQtyMap) LIKE '%Beer%'", Customer.class);
        query.getResultList().forEach(System.out::println);
    }
}